@@ -380,10 +380,6 @@ add_task(async function testAboutFind() {
 });
 
 add_task(async function testIncognitoFind() {
-  await SpecialPowers.pushPrefEnv({
-    set: [["extensions.allowPrivateBrowsingByDefault", false]],
-  });
-
   async function background() {
     await browser.test.assertRejects(
       browser.find.find("banana"),
@@ -407,7 +403,7 @@ add_task(async function testIncognitoFind() {
   let privateWin = await BrowserTestUtils.openNewBrowserWindow({
     private: true,
   });
-  await BrowserTestUtils.loadURI(
+  BrowserTestUtils.loadURI(
     privateWin.gBrowser.selectedBrowser,
     "http://example.com"
   );
@@ -427,10 +423,6 @@ add_task(async function testIncognitoFind() {
 });
 
 add_task(async function testIncognitoFindAllowed() {
-  await SpecialPowers.pushPrefEnv({
-    set: [["extensions.allowPrivateBrowsingByDefault", false]],
-  });
-
   // We're only testing we can make the calls in a private window,
   // testFind above tests full functionality.
   async function background() {
@@ -446,7 +438,7 @@ add_task(async function testIncognitoFindAllowed() {
   let privateWin = await BrowserTestUtils.openNewBrowserWindow({
     private: true,
   });
-  await BrowserTestUtils.loadURI(privateWin.gBrowser.selectedBrowser, url);
+  BrowserTestUtils.loadURI(privateWin.gBrowser.selectedBrowser, url);
   await BrowserTestUtils.browserLoaded(privateWin.gBrowser.selectedBrowser);
 
   let extension = ExtensionTestUtils.loadExtension({
