@@ -6,13 +6,13 @@
  *          focusSearch, ensureUnifinderLoaded, toggleUnifinder
  */
 
-/* import-globals-from ../../resources/content/mouseoverPreviews.js */
+/* import-globals-from ../../../mail/base/content/utilityOverlay.js */
 /* import-globals-from ../src/calFilter.js */
+/* import-globals-from item-editing/calendar-item-editing.js */
+/* import-globals-from widgets/mouseoverPreviews.js */
 /* import-globals-from calendar-command-controller.js */
 /* import-globals-from calendar-views-utils.js */
 
-/* globals goToggleToolbar */
-
 /**
  * U N I F I N D E R
  *
@@ -65,7 +65,7 @@ function getCurrentUnifinderFilter() {
  * @see calICompositeObserver
  */
 var unifinderObserver = {
-  QueryInterface: cal.generateQI([Ci.calICompositeObserver, Ci.nsIObserver, Ci.calIObserver]),
+  QueryInterface: ChromeUtils.generateQI(["calICompositeObserver", "nsIObserver", "calIObserver"]),
 
   // calIObserver:
   onStartBatch() {
@@ -89,7 +89,7 @@ var unifinderObserver = {
 
   onAddItem(aItem) {
     if (
-      cal.item.isEvent(aItem) &&
+      aItem.isEvent() &&
       !gUnifinderNeedsRefresh &&
       unifinderTreeView.mFilter.isItemInFilters(aItem)
     ) {
@@ -103,7 +103,7 @@ var unifinderObserver = {
   },
 
   onDeleteItem(aDeletedItem) {
-    if (cal.item.isEvent(aDeletedItem) && !gUnifinderNeedsRefresh) {
+    if (aDeletedItem.isEvent() && !gUnifinderNeedsRefresh) {
       this.removeItemFromTree(aDeletedItem);
     }
   },
@@ -135,7 +135,7 @@ var unifinderObserver = {
 
   onCalendarRemoved(aDeletedCalendar) {
     if (!aDeletedCalendar.getProperty("disabled")) {
-      deleteItemsFromCalendar(aDeletedCalendar);
+      removeItemsFromCalendar(aDeletedCalendar);
     }
   },
 
@@ -185,7 +185,7 @@ var unifinderObserver = {
 };
 
 /**
- * Called when the window is loaded to prepare the unifinder. This function is
+ * Called when calendar component is loaded to prepare the unifinder. This function is
  * used to add observers, event listeners, etc.
  */
 function prepareCalendarUnifinder() {
@@ -211,9 +211,9 @@ function prepareCalendarUnifinder() {
   unifinderTree.view = unifinderTreeView;
 
   // Listen for changes in the selected day, so we can update if need be
-  let viewDeck = getViewDeck();
-  viewDeck.addEventListener("dayselect", unifinderDaySelect);
-  viewDeck.addEventListener("itemselect", unifinderItemSelect, true);
+  let viewBox = getViewBox();
+  viewBox.addEventListener("dayselect", unifinderDaySelect);
+  viewBox.addEventListener("itemselect", unifinderItemSelect, true);
 
   // Set up sortDirection and sortActive, in case it persisted
   let sorted = unifinderTree.getAttribute("sort-active");
@@ -253,10 +253,10 @@ function finishCalendarUnifinder() {
   Services.prefs.removeObserver("calendar.date.format", unifinderObserver);
   Services.obs.removeObserver(unifinderObserver, "defaultTimezoneChanged");
 
-  let viewDeck = getViewDeck();
-  if (viewDeck) {
-    viewDeck.removeEventListener("dayselect", unifinderDaySelect);
-    viewDeck.removeEventListener("itemselect", unifinderItemSelect, true);
+  let viewBox = getViewBox();
+  if (viewBox) {
+    viewBox.removeEventListener("dayselect", unifinderDaySelect);
+    viewBox.removeEventListener("itemselect", unifinderItemSelect, true);
   }
 
   // Persist the sort
@@ -313,7 +313,7 @@ function unifinderDoubleClick(event) {
   let calendarEvent = unifinderTreeView.getItemFromEvent(event);
 
   if (calendarEvent) {
-    modifyEventWithDialog(calendarEvent, null, true);
+    openEventDialogForViewing(calendarEvent);
   } else {
     createEventWithDialog();
   }
@@ -389,7 +389,7 @@ function unifinderKeyPress(aEvent) {
  * Tree controller for unifinder search results
  */
 var unifinderTreeView = {
-  QueryInterface: ChromeUtils.generateQI([Ci.nsITreeView]),
+  QueryInterface: ChromeUtils.generateQI(["nsITreeView"]),
 
   // Provide a default tree that holds all the functions used here to avoid
   // cludgy if (this.tree) { this.tree.rowCountChanged(...); } constructs.
@@ -430,7 +430,7 @@ var unifinderTreeView = {
         col.setAttribute("sortDirection", this.sortDirection);
       }
     }
-    return (this.mSelectedColumn = aCol);
+    this.mSelectedColumn = aCol;
   },
 
   /**
@@ -535,7 +535,9 @@ var unifinderTreeView = {
   clearItems() {
     let oldCount = this.eventArray.length;
     this.eventArray = [];
-    this.tree.rowCountChanged(0, -oldCount);
+    if (this.tree) {
+      this.tree.rowCountChanged(0, -oldCount);
+    }
     this.calculateIndexMap();
   },
 
@@ -565,7 +567,7 @@ var unifinderTreeView = {
       this.eventIndexMap[this.eventArray[i].hashId] = i;
     }
 
-    if (!aDontInvalidate) {
+    if (this.tree && !aDontInvalidate) {
       this.tree.invalidate();
     }
   },
@@ -912,7 +914,7 @@ function addItemsFromCalendar(aCalendar, aAddItemsInternalFunc) {
     return;
   }
   let refreshListener = {
-    QueryInterface: ChromeUtils.generateQI([Ci.calIOperationListener]),
+    QueryInterface: ChromeUtils.generateQI(["calIOperationListener"]),
     mEventArray: [],
 
     onOperationComplete(aOpCalendar, aStatus, aOperationType, aId, aDateTime) {
@@ -950,7 +952,7 @@ function addItemsFromCalendar(aCalendar, aAddItemsInternalFunc) {
   );
 }
 
-function deleteItemsFromCalendar(aCalendar) {
+function removeItemsFromCalendar(aCalendar) {
   let filter = unifinderTreeView.mFilter;
   let items = unifinderTreeView.eventArray.filter(item => item.calendar.id == aCalendar.id);
 
@@ -997,6 +999,3 @@ function toggleUnifinder() {
   }
   unifinderTreeView.setSelectedItems();
 }
-
-window.addEventListener("load", prepareCalendarUnifinder, { once: true });
-window.addEventListener("unload", finishCalendarUnifinder);
