@@ -15,12 +15,3 @@ Services.scriptloader.loadSubScript(
   "chrome://mochitests/content/browser/devtools/client/framework/browser-toolbox/test/helpers-browser-toolbox.js",
   this
 );
-
-async function selectNodeFront(inspector, walker, selector, rootNode) {
-  rootNode = rootNode || walker.rootNode;
-  const nodeFront = await walker.querySelector(rootNode, selector);
-  const updated = inspector.once("inspector-updated");
-  inspector.selection.setNodeFront(nodeFront);
-  await updated;
-  return nodeFront;
-}
