@@ -10,11 +10,11 @@ const TEST_URL = URL_ROOT + "doc_inspector_delete-selected-node-01.html";
 add_task(async function() {
   const { inspector } = await openInspectorForURL(TEST_URL);
 
-  const iframe = await getNodeFront("iframe", inspector);
-  const node = await getNodeFrontInFrame("span", iframe, inspector);
-  await selectNode(node, inspector);
+  info("Select a node inside the iframe");
+  await selectNodeInFrames(["iframe", "span"], inspector);
 
   info("Removing iframe.");
+  const iframe = await getNodeFront("iframe", inspector);
   await inspector.walker.removeNode(iframe);
   await inspector.selection.once("detached-front");
 
