@@ -147,6 +147,11 @@ class ChannelEventQueue final {
   // dispatched in a new event on the current thread.
   void Resume();
 
+  void NotifyReleasingOwner() {
+    MutexAutoLock lock(mMutex);
+    mOwner = nullptr;
+  }
+
 #ifdef MOZ_DIAGNOSTIC_ASSERT_ENABLED
   bool IsEmpty() const { return mEventQueue.IsEmpty(); }
 #endif
