@@ -9,6 +9,7 @@
 #include "CParserContext.h"
 #include "nsIExpatSink.h"
 #include "nsIContentSink.h"
+#include "nsIDocShell.h"
 #include "nsParserMsgUtils.h"
 #include "nsIURL.h"
 #include "nsIUnicharInputStream.h"
@@ -19,7 +20,9 @@
 #include "nsCRT.h"
 #include "nsIConsoleService.h"
 #include "nsIScriptError.h"
+#include "nsIScriptGlobalObject.h"
 #include "nsIContentPolicy.h"
+#include "nsComponentManagerUtils.h"
 #include "nsContentPolicyUtils.h"
 #include "nsError.h"
 #include "nsXPCOMCIDInternal.h"
@@ -28,6 +31,8 @@
 #include "mozilla/BasePrincipal.h"
 #include "mozilla/IntegerTypeTraits.h"
 #include "mozilla/NullPrincipal.h"
+#include "mozilla/Telemetry.h"
+#include "mozilla/TelemetryComms.h"
 
 #include "mozilla/Logging.h"
 
@@ -328,8 +333,8 @@ void nsExpatDriver::HandleStartElementForSystemPrincipal(
     error.AppendLiteral("> created from entity value.");
 
     nsContentUtils::ReportToConsoleNonLocalized(
-        error, nsIScriptError::warningFlag, NS_LITERAL_CSTRING("XML Document"),
-        doc, nullptr, EmptyString(), lineNumber, colNumber);
+        error, nsIScriptError::warningFlag, "XML Document"_ns, doc, nullptr,
+        u""_ns, lineNumber, colNumber);
   }
 }
 
@@ -655,7 +660,7 @@ nsresult nsExpatDriver::OpenInputStreamFromExternalDTD(const char16_t* aFPIStr,
     localURI.swap(uri);
     rv = NS_NewChannel(getter_AddRefs(channel), uri,
                        nsContentUtils::GetSystemPrincipal(),
-                       nsILoadInfo::SEC_ALLOW_CROSS_ORIGIN_DATA_IS_NULL,
+                       nsILoadInfo::SEC_ALLOW_CROSS_ORIGIN_SEC_CONTEXT_IS_NULL,
                        nsIContentPolicy::TYPE_DTD);
     NS_ENSURE_SUCCESS(rv, rv);
   } else {
@@ -671,20 +676,22 @@ nsresult nsExpatDriver::OpenInputStreamFromExternalDTD(const char16_t* aFPIStr,
         if (doc->SkipDTDSecurityChecks()) {
           policyType = nsIContentPolicy::TYPE_INTERNAL_FORCE_ALLOWED_DTD;
         }
-        rv = NS_NewChannel(getter_AddRefs(channel), uri, doc,
-                           nsILoadInfo::SEC_ALLOW_CROSS_ORIGIN_DATA_INHERITS |
-                               nsILoadInfo::SEC_ALLOW_CHROME,
-                           policyType);
+        rv = NS_NewChannel(
+            getter_AddRefs(channel), uri, doc,
+            nsILoadInfo::SEC_ALLOW_CROSS_ORIGIN_INHERITS_SEC_CONTEXT |
+                nsILoadInfo::SEC_ALLOW_CHROME,
+            policyType);
         NS_ENSURE_SUCCESS(rv, rv);
       }
     }
     if (!channel) {
       nsCOMPtr<nsIPrincipal> nullPrincipal =
           mozilla::NullPrincipal::CreateWithoutOriginAttributes();
-      rv = NS_NewChannel(getter_AddRefs(channel), uri, nullPrincipal,
-                         nsILoadInfo::SEC_ALLOW_CROSS_ORIGIN_DATA_INHERITS |
-                             nsILoadInfo::SEC_ALLOW_CHROME,
-                         policyType);
+      rv = NS_NewChannel(
+          getter_AddRefs(channel), uri, nullPrincipal,
+          nsILoadInfo::SEC_ALLOW_CROSS_ORIGIN_INHERITS_SEC_CONTEXT |
+              nsILoadInfo::SEC_ALLOW_CHROME,
+          policyType);
       NS_ENSURE_SUCCESS(rv, rv);
     }
   }
@@ -694,7 +701,7 @@ nsresult nsExpatDriver::OpenInputStreamFromExternalDTD(const char16_t* aFPIStr,
   NS_ENSURE_SUCCESS(rv, rv);
   CopyUTF8toUTF16(absURL, aAbsURL);
 
-  channel->SetContentType(NS_LITERAL_CSTRING("application/xml"));
+  channel->SetContentType("application/xml"_ns);
   return channel->Open(aStream);
 }
 
@@ -753,6 +760,7 @@ nsresult nsExpatDriver::HandleError() {
   if (mOriginalSink) {
     doc = do_QueryInterface(mOriginalSink->GetTarget());
   }
+
   bool spoofEnglish =
       nsContentUtils::SpoofLocaleEnglish() && (!doc || !doc->AllowsL10n());
   nsParserMsgUtils::GetLocalizedStringByID(
@@ -814,6 +822,39 @@ nsresult nsExpatDriver::HandleError() {
   nsAutoString sourceText(mLastLine);
   AppendErrorPointer(colNumber, mLastLine.get(), sourceText);
 
+  if (doc && nsContentUtils::IsChromeDoc(doc)) {
+    nsCString path = doc->GetDocumentURI()->GetSpecOrDefault();
+    nsCOMPtr<nsISupports> container = doc->GetContainer();
+    nsCOMPtr<nsIDocShell> docShell = do_QueryInterface(container);
+    nsCString docShellDestroyed("unknown"_ns);
+    if (docShell) {
+      bool destroyed = false;
+      docShell->IsBeingDestroyed(&destroyed);
+      docShellDestroyed.Assign(destroyed ? "true"_ns : "false"_ns);
+    }
+
+    mozilla::Maybe<nsTArray<mozilla::Telemetry::EventExtraEntry>> extra =
+        mozilla::Some<nsTArray<mozilla::Telemetry::EventExtraEntry>>({
+            mozilla::Telemetry::EventExtraEntry{"error_code"_ns,
+                                                nsPrintfCString("%u", code)},
+            mozilla::Telemetry::EventExtraEntry{
+                "location"_ns, nsPrintfCString("%u:%u", lineNumber, colNumber)},
+            mozilla::Telemetry::EventExtraEntry{
+                "last_line"_ns, NS_ConvertUTF16toUTF8(mLastLine)},
+            mozilla::Telemetry::EventExtraEntry{
+                "last_line_len"_ns, nsPrintfCString("%zu", mLastLine.Length())},
+            mozilla::Telemetry::EventExtraEntry{
+                "hidden"_ns, doc->Hidden() ? "true"_ns : "false"_ns},
+            mozilla::Telemetry::EventExtraEntry{"destroyed"_ns,
+                                                docShellDestroyed},
+        });
+
+    mozilla::Telemetry::SetEventRecordingEnabled("ysod"_ns, true);
+    mozilla::Telemetry::RecordEvent(
+        mozilla::Telemetry::EventID::Ysod_Shown_Ysod, mozilla::Some(path),
+        extra);
+  }
+
   // Try to create and initialize the script error.
   nsCOMPtr<nsIScriptError> serr(do_CreateInstance(NS_SCRIPTERROR_CONTRACTID));
   nsresult rv = NS_ERROR_FAILURE;
