@@ -18,6 +18,8 @@
 #include "mozilla/Telemetry.h"
 #include "mozilla/Unused.h"
 #include "nsContentUtils.h"
+#include "nsIChannel.h"
+#include "nsIHttpChannel.h"
 #include "nsIHttpChannelInternal.h"
 #include "nsIPrompt.h"
 #include "nsIProtocolProxyService.h"
@@ -43,7 +45,6 @@
 #include "TrustOverrideUtils.h"
 #include "TrustOverride-SymantecData.inc"
 #include "TrustOverride-AppleGoogleDigiCertData.inc"
-#include "TrustOverride-TestImminentDistrustData.inc"
 
 using namespace mozilla;
 using namespace mozilla::pkix;
@@ -199,9 +200,8 @@ nsresult OCSPRequest::GetResponse(/*out*/ Vector<uint8_t>& response) {
   return NS_OK;
 }
 
-static NS_NAMED_LITERAL_CSTRING(OCSP_REQUEST_MIME_TYPE,
-                                "application/ocsp-request");
-static NS_NAMED_LITERAL_CSTRING(OCSP_REQUEST_METHOD, "POST");
+static constexpr auto OCSP_REQUEST_MIME_TYPE = "application/ocsp-request"_ns;
+static constexpr auto OCSP_REQUEST_METHOD = "POST"_ns;
 
 NS_IMETHODIMP
 OCSPRequest::Run() {
@@ -248,7 +248,7 @@ OCSPRequest::Run() {
                        nullptr,  // aLoadingNode
                        nsContentUtils::GetSystemPrincipal(),
                        nullptr,  // aTriggeringPrincipal
-                       nsILoadInfo::SEC_ALLOW_CROSS_ORIGIN_DATA_IS_NULL,
+                       nsILoadInfo::SEC_ALLOW_CROSS_ORIGIN_SEC_CONTEXT_IS_NULL,
                        nsIContentPolicy::TYPE_OTHER, getter_AddRefs(channel));
   if (NS_FAILED(rv)) {
     return NotifyDone(rv, lock);
@@ -261,9 +261,10 @@ OCSPRequest::Run() {
     priorityChannel->AdjustPriority(nsISupportsPriority::PRIORITY_HIGHEST);
   }
 
-  channel->SetLoadFlags(nsIRequest::LOAD_ANONYMOUS |
-                        nsIChannel::LOAD_BYPASS_SERVICE_WORKER |
-                        nsIChannel::LOAD_BYPASS_URL_CLASSIFIER);
+  channel->SetLoadFlags(
+      nsIRequest::LOAD_ANONYMOUS | nsIRequest::LOAD_BYPASS_CACHE |
+      nsIRequest::INHIBIT_CACHING | nsIChannel::LOAD_BYPASS_SERVICE_WORKER |
+      nsIChannel::LOAD_BYPASS_URL_CLASSIFIER);
 
   nsCOMPtr<nsILoadInfo> loadInfo = channel->LoadInfo();
 
@@ -307,7 +308,7 @@ OCSPRequest::Run() {
   if (NS_FAILED(rv)) {
     return NotifyDone(rv, lock);
   }
-  // Do not use SPDY for internal security operations. It could result
+  // Do not use SPDY or HTTP3 for internal security operations. It could result
   // in the silent upgrade to ssl, which in turn could require an SSL
   // operation to fulfill something like an OCSP fetch, which is an
   // endless loop.
@@ -319,6 +320,14 @@ OCSPRequest::Run() {
   if (NS_FAILED(rv)) {
     return NotifyDone(rv, lock);
   }
+  rv = internalChannel->SetAllowHttp3(false);
+  if (NS_FAILED(rv)) {
+    return NotifyDone(rv, lock);
+  }
+  rv = internalChannel->SetIsOCSP(true);
+  if (NS_FAILED(rv)) {
+    return NotifyDone(rv, lock);
+  }
   nsCOMPtr<nsIHttpChannel> hchan = do_QueryInterface(channel);
   if (!hchan) {
     return NotifyDone(NS_ERROR_FAILURE, lock);
@@ -570,16 +579,12 @@ void PK11PasswordPromptRunnable::RunOnTargetThread() {
 
   nsAutoString promptString;
   if (PK11_IsInternal(mSlot)) {
-    rv = GetPIPNSSBundleString("CertPassPromptDefault", promptString);
+    rv = GetPIPNSSBundleString("CertPasswordPromptDefault", promptString);
   } else {
     AutoTArray<nsString, 1> formatStrings = {
         NS_ConvertUTF8toUTF16(PK11_GetTokenName(mSlot))};
     rv = PIPBundleFormatStringFromName("CertPasswordPrompt", formatStrings,
                                        promptString);
-    if (NS_FAILED(rv)) {
-      rv = PIPBundleFormatStringFromName("CertPassPrompt", formatStrings,
-                                         promptString);
-    }
   }
   if (NS_FAILED(rv)) {
     return;
@@ -611,35 +616,35 @@ nsCString getKeaGroupName(uint32_t aKeaGroup) {
   nsCString groupName;
   switch (aKeaGroup) {
     case ssl_grp_ec_secp256r1:
-      groupName = NS_LITERAL_CSTRING("P256");
+      groupName = "P256"_ns;
       break;
     case ssl_grp_ec_secp384r1:
-      groupName = NS_LITERAL_CSTRING("P384");
+      groupName = "P384"_ns;
       break;
     case ssl_grp_ec_secp521r1:
-      groupName = NS_LITERAL_CSTRING("P521");
+      groupName = "P521"_ns;
       break;
     case ssl_grp_ec_curve25519:
-      groupName = NS_LITERAL_CSTRING("x25519");
+      groupName = "x25519"_ns;
       break;
     case ssl_grp_ffdhe_2048:
-      groupName = NS_LITERAL_CSTRING("FF 2048");
+      groupName = "FF 2048"_ns;
       break;
     case ssl_grp_ffdhe_3072:
-      groupName = NS_LITERAL_CSTRING("FF 3072");
+      groupName = "FF 3072"_ns;
       break;
     case ssl_grp_none:
-      groupName = NS_LITERAL_CSTRING("none");
+      groupName = "none"_ns;
       break;
     case ssl_grp_ffdhe_custom:
-      groupName = NS_LITERAL_CSTRING("custom");
+      groupName = "custom"_ns;
       break;
     // All other groups are not enabled in Firefox. See namedGroups in
     // nsNSSIOLayer.cpp.
     default:
       // This really shouldn't happen!
       MOZ_ASSERT_UNREACHABLE("Invalid key exchange group.");
-      groupName = NS_LITERAL_CSTRING("unknown group");
+      groupName = "unknown group"_ns;
   }
   return groupName;
 }
@@ -648,50 +653,50 @@ nsCString getSignatureName(uint32_t aSignatureScheme) {
   nsCString signatureName;
   switch (aSignatureScheme) {
     case ssl_sig_none:
-      signatureName = NS_LITERAL_CSTRING("none");
+      signatureName = "none"_ns;
       break;
     case ssl_sig_rsa_pkcs1_sha1:
-      signatureName = NS_LITERAL_CSTRING("RSA-PKCS1-SHA1");
+      signatureName = "RSA-PKCS1-SHA1"_ns;
       break;
     case ssl_sig_rsa_pkcs1_sha256:
-      signatureName = NS_LITERAL_CSTRING("RSA-PKCS1-SHA256");
+      signatureName = "RSA-PKCS1-SHA256"_ns;
       break;
     case ssl_sig_rsa_pkcs1_sha384:
-      signatureName = NS_LITERAL_CSTRING("RSA-PKCS1-SHA384");
+      signatureName = "RSA-PKCS1-SHA384"_ns;
       break;
     case ssl_sig_rsa_pkcs1_sha512:
-      signatureName = NS_LITERAL_CSTRING("RSA-PKCS1-SHA512");
+      signatureName = "RSA-PKCS1-SHA512"_ns;
       break;
     case ssl_sig_ecdsa_secp256r1_sha256:
-      signatureName = NS_LITERAL_CSTRING("ECDSA-P256-SHA256");
+      signatureName = "ECDSA-P256-SHA256"_ns;
       break;
     case ssl_sig_ecdsa_secp384r1_sha384:
-      signatureName = NS_LITERAL_CSTRING("ECDSA-P384-SHA384");
+      signatureName = "ECDSA-P384-SHA384"_ns;
       break;
     case ssl_sig_ecdsa_secp521r1_sha512:
-      signatureName = NS_LITERAL_CSTRING("ECDSA-P521-SHA512");
+      signatureName = "ECDSA-P521-SHA512"_ns;
       break;
     case ssl_sig_rsa_pss_sha256:
-      signatureName = NS_LITERAL_CSTRING("RSA-PSS-SHA256");
+      signatureName = "RSA-PSS-SHA256"_ns;
       break;
     case ssl_sig_rsa_pss_sha384:
-      signatureName = NS_LITERAL_CSTRING("RSA-PSS-SHA384");
+      signatureName = "RSA-PSS-SHA384"_ns;
       break;
     case ssl_sig_rsa_pss_sha512:
-      signatureName = NS_LITERAL_CSTRING("RSA-PSS-SHA512");
+      signatureName = "RSA-PSS-SHA512"_ns;
       break;
     case ssl_sig_ecdsa_sha1:
-      signatureName = NS_LITERAL_CSTRING("ECDSA-SHA1");
+      signatureName = "ECDSA-SHA1"_ns;
       break;
     case ssl_sig_rsa_pkcs1_sha1md5:
-      signatureName = NS_LITERAL_CSTRING("RSA-PKCS1-SHA1MD5");
+      signatureName = "RSA-PKCS1-SHA1MD5"_ns;
       break;
     // All other groups are not enabled in Firefox. See sEnabledSignatureSchemes
     // in nsNSSIOLayer.cpp.
     default:
       // This really shouldn't happen!
       MOZ_ASSERT_UNREACHABLE("Invalid signature scheme.");
-      signatureName = NS_LITERAL_CSTRING("unknown signature");
+      signatureName = "unknown signature"_ns;
   }
   return signatureName;
 }
@@ -721,11 +726,7 @@ static void PreliminaryHandshakeDone(PRFileDesc* fd) {
       infoObject->SetKEAKeyBits(channelInfo.keaKeyBits);
       infoObject->SetMACAlgorithmUsed(cipherInfo.macAlgorithm);
       infoObject->mIsDelegatedCredential = channelInfo.peerDelegCred;
-
-      if (infoObject->mIsDelegatedCredential) {
-        Telemetry::ScalarAdd(
-            Telemetry::ScalarID::SECURITY_TLS_DELEGATED_CREDENTIALS_TXN, 1);
-      }
+      infoObject->mIsAcceptedEch = channelInfo.echAccepted;
     }
   }
 
@@ -840,48 +841,27 @@ SECStatus CanFalseStartCallback(PRFileDesc* fd, void* client_data,
 
 static void AccumulateNonECCKeySize(Telemetry::HistogramID probe,
                                     uint32_t bits) {
-  unsigned int value =
-      bits < 512
-          ? 1
-          : bits == 512
-                ? 2
-                : bits < 768
-                      ? 3
-                      : bits == 768
-                            ? 4
-                            : bits < 1024
-                                  ? 5
-                                  : bits == 1024
-                                        ? 6
-                                        : bits < 1280
-                                              ? 7
-                                              : bits == 1280
-                                                    ? 8
-                                                    : bits < 1536
-                                                          ? 9
-                                                          : bits == 1536
-                                                                ? 10
-                                                                : bits < 2048
-                                                                      ? 11
-                                                                      : bits == 2048
-                                                                            ? 12
-                                                                            : bits < 3072
-                                                                                  ? 13
-                                                                                  : bits == 3072
-                                                                                        ? 14
-                                                                                        : bits < 4096
-                                                                                              ? 15
-                                                                                              : bits == 4096
-                                                                                                    ? 16
-                                                                                                    : bits < 8192
-                                                                                                          ? 17
-                                                                                                          : bits == 8192
-                                                                                                                ? 18
-                                                                                                                : bits < 16384
-                                                                                                                      ? 19
-                                                                                                                      : bits == 16384
-                                                                                                                            ? 20
-                                                                                                                            : 0;
+  unsigned int value = bits < 512      ? 1
+                       : bits == 512   ? 2
+                       : bits < 768    ? 3
+                       : bits == 768   ? 4
+                       : bits < 1024   ? 5
+                       : bits == 1024  ? 6
+                       : bits < 1280   ? 7
+                       : bits == 1280  ? 8
+                       : bits < 1536   ? 9
+                       : bits == 1536  ? 10
+                       : bits < 2048   ? 11
+                       : bits == 2048  ? 12
+                       : bits < 3072   ? 13
+                       : bits == 3072  ? 14
+                       : bits < 4096   ? 15
+                       : bits == 4096  ? 16
+                       : bits < 8192   ? 17
+                       : bits == 8192  ? 18
+                       : bits < 16384  ? 19
+                       : bits == 16384 ? 20
+                                       : 0;
   Telemetry::Accumulate(probe, value);
 }
 
@@ -892,12 +872,11 @@ static void AccumulateNonECCKeySize(Telemetry::HistogramID probe,
 // named curves for a given size (e.g. secp256k1 vs. secp256r1). We punt on
 // that for now. See also NSS bug 323674.
 static void AccumulateECCCurve(Telemetry::HistogramID probe, uint32_t bits) {
-  unsigned int value =
-      bits == 255 ? 29                                            // Curve25519
-                  : bits == 256 ? 23                              // P-256
-                                : bits == 384 ? 24                // P-384
-                                              : bits == 521 ? 25  // P-521
-                                                            : 0;  // Unknown
+  unsigned int value = bits == 255   ? 29  // Curve25519
+                       : bits == 256 ? 23  // P-256
+                       : bits == 384 ? 24  // P-384
+                       : bits == 521 ? 25  // P-521
+                                     : 0;  // Unknown
   Telemetry::Accumulate(probe, value);
 }
 
@@ -1117,23 +1096,20 @@ static void RebuildVerifiedCertificateInformation(PRFileDesc* fd,
     flags |= CertVerifier::FLAG_TLS_IGNORE_STATUS_REQUEST;
   }
 
-  SECOidTag evOidPolicy;
+  EVStatus evStatus;
   CertificateTransparencyInfo certificateTransparencyInfo;
   UniqueCERTCertList builtChain;
-  const bool saveIntermediates = false;
   bool isBuiltCertChainRootBuiltInRoot = false;
   mozilla::pkix::Result rv = certVerifier->VerifySSLServerCert(
       cert, mozilla::pkix::Now(), infoObject, infoObject->GetHostName(),
       builtChain, flags, maybePeerCertsBytes, stapledOCSPResponse,
       sctsFromTLSExtension, Nothing(), infoObject->GetOriginAttributes(),
-      saveIntermediates, &evOidPolicy,
+      &evStatus,
       nullptr,  // OCSP stapling telemetry
       nullptr,  // key size telemetry
       nullptr,  // SHA-1 telemetry
       nullptr,  // pinning telemetry
-      &certificateTransparencyInfo,
-      nullptr,  // CRLite telemetry,
-      &isBuiltCertChainRootBuiltInRoot);
+      &certificateTransparencyInfo, &isBuiltCertChainRootBuiltInRoot);
 
   if (rv != Success) {
     MOZ_LOG(gPIPNSSLog, LogLevel::Debug,
@@ -1141,7 +1117,7 @@ static void RebuildVerifiedCertificateInformation(PRFileDesc* fd,
   }
 
   RefPtr<nsNSSCertificate> nssc(nsNSSCertificate::Create(cert.get()));
-  if (rv == Success && evOidPolicy != SEC_OID_UNKNOWN) {
+  if (rv == Success && evStatus == EVStatus::EV) {
     MOZ_LOG(gPIPNSSLog, LogLevel::Debug,
             ("HandshakeCallback using NEW cert %p (is EV)", nssc.get()));
     infoObject->SetServerCert(nssc, EVStatus::EV);
@@ -1164,101 +1140,6 @@ static void RebuildVerifiedCertificateInformation(PRFileDesc* fd,
   }
 }
 
-nsresult IsCertificateDistrustImminent(
-    const nsTArray<RefPtr<nsIX509Cert>>& aCertArray,
-    /* out */ bool& isDistrusted) {
-  if (aCertArray.IsEmpty()) {
-    return NS_ERROR_INVALID_ARG;
-  }
-
-  nsCOMPtr<nsIX509Cert> rootCert;
-  nsTArray<RefPtr<nsIX509Cert>> intCerts;
-  nsCOMPtr<nsIX509Cert> eeCert;
-
-  nsresult rv = nsNSSCertificate::SegmentCertificateChain(aCertArray, rootCert,
-                                                          intCerts, eeCert);
-  if (NS_FAILED(rv)) {
-    return rv;
-  }
-
-  // Check the test certificate condition first; this is a special certificate
-  // that gets the 'imminent distrust' treatment; this is so that the distrust
-  // UX code does not become stale, as it will need regular use. See Bug 1409257
-  // for context. Please do not remove this when adjusting the rest of the
-  // method.
-  UniqueCERTCertificate nssEECert(eeCert->GetCert());
-  if (!nssEECert) {
-    return NS_ERROR_FAILURE;
-  }
-  isDistrusted =
-      CertDNIsInList(nssEECert.get(), TestImminentDistrustEndEntityDNs);
-  if (isDistrusted) {
-    // Exit early
-    return NS_OK;
-  }
-
-  UniqueCERTCertificate nssRootCert(rootCert->GetCert());
-  if (!nssRootCert) {
-    return NS_ERROR_FAILURE;
-  }
-
-  // Proceed with the Symantec imminent distrust algorithm. This algorithm is
-  // to be removed in Firefox 63, when the validity period check will also be
-  // removed from the code in NSSCertDBTrustDomain.
-  if (CertDNIsInList(nssRootCert.get(), RootSymantecDNs)) {
-    static const PRTime NULL_TIME = 0;
-
-    rv = CheckForSymantecDistrust(intCerts, eeCert, NULL_TIME,
-                                  RootAppleAndGoogleSPKIs, isDistrusted);
-    if (NS_FAILED(rv)) {
-      return rv;
-    }
-  }
-  return NS_OK;
-}
-
-static void RebuildCertificateInfoFromSSLTokenCache(
-    nsNSSSocketInfo* aInfoObject) {
-  MOZ_ASSERT(aInfoObject);
-
-  if (!aInfoObject) {
-    return;
-  }
-
-  nsAutoCString key;
-  aInfoObject->GetPeerId(key);
-  mozilla::net::SessionCacheInfo info;
-  if (!mozilla::net::SSLTokensCache::GetSessionCacheInfo(key, info)) {
-    MOZ_LOG(
-        gPIPNSSLog, LogLevel::Debug,
-        ("RebuildCertificateInfoFromSSLTokenCache cannot find cached info."));
-    return;
-  }
-
-  RefPtr<nsNSSCertificate> nssc = nsNSSCertificate::ConstructFromDER(
-      BitwiseCast<char*, uint8_t*>(info.mServerCertBytes.Elements()),
-      info.mServerCertBytes.Length());
-  if (!nssc) {
-    MOZ_LOG(gPIPNSSLog, LogLevel::Debug,
-            ("RebuildCertificateInfoFromSSLTokenCache failed to construct "
-             "server cert"));
-    return;
-  }
-
-  aInfoObject->SetServerCert(nssc, info.mEVStatus);
-  aInfoObject->SetCertificateTransparencyStatus(
-      info.mCertificateTransparencyStatus);
-  if (info.mSucceededCertChainBytes) {
-    aInfoObject->SetSucceededCertChain(
-        std::move(*info.mSucceededCertChainBytes));
-  }
-
-  if (info.mIsBuiltCertChainRootBuiltInRoot) {
-    aInfoObject->SetIsBuiltCertChainRootBuiltInRoot(
-        *info.mIsBuiltCertChainRootBuiltInRoot);
-  }
-}
-
 void HandshakeCallback(PRFileDesc* fd, void* client_data) {
   SECStatus rv;
 
@@ -1397,28 +1278,12 @@ void HandshakeCallback(PRFileDesc* fd, void* client_data) {
             ("HandshakeCallback KEEPING existing cert\n"));
   } else {
     if (StaticPrefs::network_ssl_tokens_cache_enabled()) {
-      RebuildCertificateInfoFromSSLTokenCache(infoObject);
-      infoObject->NoteSessionResumptionTime(true);
+      infoObject->RebuildCertificateInfoFromSSLTokenCache();
     } else {
       RebuildVerifiedCertificateInformation(fd, infoObject);
-      infoObject->NoteSessionResumptionTime(false);
     }
   }
 
-  nsTArray<RefPtr<nsIX509Cert>> succeededCertArray;
-  // The list could be empty. Bug 731478 will reduce the incidence of empty
-  // succeeded cert chains through better caching.
-  nsresult srv = infoObject->GetSucceededCertChain(succeededCertArray);
-
-  bool distrustImminent;
-  if (NS_SUCCEEDED(srv)) {
-    srv = IsCertificateDistrustImminent(succeededCertArray, distrustImminent);
-  }
-
-  if (NS_SUCCEEDED(srv) && distrustImminent) {
-    state |= nsIWebProgressListener::STATE_CERT_DISTRUST_IMMINENT;
-  }
-
   bool domainMismatch;
   bool untrusted;
   bool notValidAtThisTime;
