@@ -83,6 +83,7 @@ const { fxAccounts } = ChromeUtils.import(
 function getEngineModules() {
   let result = {
     Addons: { module: "addons.js", symbol: "AddonsEngine" },
+    Bookmarks: { module: "bookmarks.js", symbol: "BookmarksEngine" },
     Form: { module: "forms.js", symbol: "FormEngine" },
     History: { module: "history.js", symbol: "HistoryEngine" },
     Password: { module: "passwords.js", symbol: "PasswordEngine" },
@@ -91,22 +92,16 @@ function getEngineModules() {
   };
   if (Svc.Prefs.get("engine.addresses.available", false)) {
     result.Addresses = {
-      module: "resource://formautofill/FormAutofillSync.jsm",
+      module: "resource://autofill/FormAutofillSync.jsm",
       symbol: "AddressesEngine",
     };
   }
   if (Svc.Prefs.get("engine.creditcards.available", false)) {
     result.CreditCards = {
-      module: "resource://formautofill/FormAutofillSync.jsm",
+      module: "resource://autofill/FormAutofillSync.jsm",
       symbol: "CreditCardsEngine",
     };
   }
-  result.Bookmarks = {
-    module: "bookmarks.js",
-    controllingPref: "services.sync.engine.bookmarks.buffer",
-    whenFalse: "BookmarksEngine",
-    whenTrue: "BufferedBookmarksEngine",
-  };
   result["Extension-Storage"] = {
     module: "extension-storage.js",
     controllingPref: "webextensions.storage.sync.kinto",
@@ -547,8 +542,8 @@ Sync11Service.prototype = {
   },
 
   QueryInterface: ChromeUtils.generateQI([
-    Ci.nsIObserver,
-    Ci.nsISupportsWeakReference,
+    "nsIObserver",
+    "nsISupportsWeakReference",
   ]),
 
   observe(subject, topic, data) {
@@ -1571,8 +1566,8 @@ Sync11Service.prototype = {
    * Wipe all remote user data by wiping the server then telling each remote
    * client to wipe itself.
    *
-   * @param engines [optional]
-   *        Array of engine names to wipe. If not given, all engines are used.
+   * @param engines
+   *        Array of engine names to wipe.
    */
   async wipeRemote(engines) {
     try {
@@ -1584,13 +1579,8 @@ Sync11Service.prototype = {
 
       // Only wipe the engines provided.
       let extra = { reason: "wipe-remote" };
-      if (engines) {
-        for (const e of engines) {
-          await this.clientsEngine.sendCommand("wipeEngine", [e], null, extra);
-        }
-      } else {
-        // Tell the remote machines to wipe themselves.
-        await this.clientsEngine.sendCommand("wipeAll", [], null, extra);
+      for (const e of engines) {
+        await this.clientsEngine.sendCommand("wipeEngine", [e], null, extra);
       }
 
       // Make sure the changed clients get updated.
