@@ -16,28 +16,38 @@ from mozbuild.base import MachCommandBase
 
 def is_osx_10_10_or_greater(cls):
     import platform
+
     release = platform.mac_ver()[0]
-    return release and StrictVersion(release) >= StrictVersion('10.10')
+    return release and StrictVersion(release) >= StrictVersion("10.10")
 
 
 @CommandProvider
 class MachCommands(MachCommandBase):
-    '''
+    """
     Get system power consumption and related measurements.
-    '''
-    @Command('power', category='misc',
-             conditions=[is_osx_10_10_or_greater],
-             description='Get system power consumption and related measurements for '
-             'all running browsers. Available only on Mac OS X 10.10 and above. '
-             'Requires root access.')
-    @CommandArgument('-i', '--interval', type=int, default=30000,
-                     help='The sample period, measured in milliseconds. Defaults to 30000.')
-    def power(self, interval):
+    """
+
+    @Command(
+        "power",
+        category="misc",
+        conditions=[is_osx_10_10_or_greater],
+        description="Get system power consumption and related measurements for "
+        "all running browsers. Available only on Mac OS X 10.10 and above. "
+        "Requires root access.",
+    )
+    @CommandArgument(
+        "-i",
+        "--interval",
+        type=int,
+        default=30000,
+        help="The sample period, measured in milliseconds. Defaults to 30000.",
+    )
+    def power(self, command_context, interval):
         import os
         import re
         import subprocess
 
-        rapl = os.path.join(self.topobjdir, 'dist', 'bin', 'rapl')
+        rapl = os.path.join(self.topobjdir, "dist", "bin", "rapl")
 
         interval = str(interval)
 
@@ -46,23 +56,31 @@ class MachCommands(MachCommandBase):
         # doesn't start measuring while |powermetrics| is waiting for the root
         # password to be entered.
         try:
-            subprocess.check_call(['sudo', 'true'])
+            subprocess.check_call(["sudo", "true"])
         except Exception:
-            print('\nsudo failed; aborting')
+            print("\nsudo failed; aborting")
             return 1
 
         # This runs rapl in the background because nothing in this script
         # depends on the output. This is good because we want |rapl| and
         # |powermetrics| to run at the same time.
-        subprocess.Popen([rapl, '-n', '1', '-i', interval])
+        subprocess.Popen([rapl, "-n", "1", "-i", interval])
 
-        lines = subprocess.check_output(['sudo', 'powermetrics',
-                                         '--samplers', 'tasks',
-                                         '--show-process-coalition',
-                                         '--show-process-gpu',
-                                         '-n', '1',
-                                         '-i', interval],
-                                        universal_newlines=True)
+        lines = subprocess.check_output(
+            [
+                "sudo",
+                "powermetrics",
+                "--samplers",
+                "tasks",
+                "--show-process-coalition",
+                "--show-process-gpu",
+                "-n",
+                "1",
+                "-i",
+                interval,
+            ],
+            universal_newlines=True,
+        )
 
         # When run with --show-process-coalition, |powermetrics| groups outputs
         # into process coalitions, each of which has a leader.
@@ -131,7 +149,10 @@ class MachCommands(MachCommandBase):
             #
             # - 'kernel' is for the kernel.
             #
-            if re.search(r'(^Name|firefox|plugin-container|Safari\b|WebKit|Chrome|Terminal|WindowServer|kernel)', line):  # NOQA: E501
+            if re.search(
+                r"(^Name|firefox|plugin-container|Safari\b|WebKit|Chrome|Terminal|WindowServer|kernel)",  # NOQA: E501
+                line,
+            ):
                 print(line)
 
         return 0
